#ifndef DISKCACHE__MOUNT_POINT_CACHE_HH
#define DISKCACHE__MOUNT_POINT_CACHE_HH

#define ENABLE_MOUNT_POINT_CACHE 1

#include "config.h"

#include <string>
#include <vector>

#include "general/SharedPtr.hh"
#include "general/unordered_map.hh"

#include "FilenameHandler.hh"

namespace diskCache
{
  class DirectoryInfo;
  class Query;
  class QueryResults;

  class MountPointCache
  {
  public:
    class Info
    {
    public:
      typedef FilenameHandler::container_info_key_type	key_type;
      typedef FilenameHandler::container_info_key_base_type
      key_base_type;

      typedef General::SharedConstPtr< DirectoryInfo > dir_info_type;
      typedef dir_info_type directory_type;// Backwards compatability

      Info( key_type Key,
	    directory_type DirInfo
	    );

      virtual ~Info( );

      key_type Key( ) const;

      directory_type Directory( ) const;

    private:
      //-----------------------------------------------------------------
      /// \brief Directory information where data was discovered
      //-----------------------------------------------------------------
      key_type		m_key;
      directory_type	m_directory_info;
    };

    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    //-------------------------------------------------------------------
    /// \brief Desctructor
    //-------------------------------------------------------------------
    virtual ~MountPointCache( );

#if ENABLE_MOUNT_POINT_CACHE && 0
    //-------------------------------------------------------------------
    /// \brief Search for information within the cache
    ///	
    /// \param[in] SearchParams
    ///     The search parameters describing the data to find.
    /// \param[out] Results
    ///     The string representation of the data found.
    //-------------------------------------------------------------------
    virtual void Find( const Query& SearchParams,
		       std::string& Results ) = 0;
#endif /* ENABLE_MOUNT_POINT_CACHE */

    //-------------------------------------------------------------------
    /// \brief Search for information within the cache
    ///	
    /// \param[in] SearchParams
    ///     The search parameters describing the data to find.
    /// \param[out] Results
    ///     The data found.
    //-------------------------------------------------------------------
    virtual void Find( const Query& SearchParams,
		       QueryResults& Results ) = 0;

    //-------------------------------------------------------------------
    /// \brief Register DirectoryInfo information
    //-------------------------------------------------------------------
    virtual void Register( General::SharedPtr< FilenameHandler::Hash > DirectoryInfoHash ) = 0;

    //-------------------------------------------------------------------
    /// \brief Unregister DirectoryInfo information
    //-------------------------------------------------------------------
    virtual void Unregister( General::SharedPtr< FilenameHandler::Hash > DirectoryInfoHash ) = 0;

    //-------------------------------------------------------------------
    /// \brief Check the state of emptiness.
    //-------------------------------------------------------------------
    virtual bool empty( ) const = 0;

  };

  inline MountPointCache::Info::
  Info( key_type Key,
	directory_type DirInfo )
    : m_key( Key ),
      m_directory_info( DirInfo )
  {
  }

  inline MountPointCache::Info::key_type MountPointCache::Info::
  Key( ) const
  {
    return m_key;
  }

  inline MountPointCache::Info::directory_type MountPointCache::Info::
  Directory( ) const
  {
    return m_directory_info;
  }

} // namespace - diskCache

#endif /* DISKCACHE__MOUNT_POINT_CACHE_HH */
