#ifndef FRAME_C_CC
#define FRAME_C_CC

#include <fstream>
#include <sstream>

#include "framecpp/FrameCPP.hh"

#include "framec/FrameC.h"

#include "FrameCInternal.hh"

using namespace FrameCPP;

namespace {
  const char* type_name( FrameC::Pointers::pointer_type Type )
  {
    using FrameC::Pointers;

    switch( Type )
    {
    case Pointers::FR_DETECTOR:
      return Pointers::Name< Pointers::FR_DETECTOR >( );
      break;
    case Pointers::FR_FRAME:
      return Pointers::Name< Pointers::FR_FRAME >( );
      break;
    case Pointers::FR_TOC:
      return Pointers::Name< Pointers::FR_TOC >( );
      break;
    case Pointers::STREAM:
      return Pointers::Name< Pointers::STREAM >( );
      break;
    case Pointers::STREAM_FILE_INPUT:
      return Pointers::Name< Pointers::STREAM_FILE_INPUT >( );
      break;
    case Pointers::STREAM_FILE_OUTPUT:
      return Pointers::Name< Pointers::STREAM_FILE_OUTPUT >( );
      break;
    }
    return "unknown pointer type";
  }
}

/*=======================================================================
 * Implementation of functions for general consumption
 *=======================================================================*/

INT_2U FrameCLibraryMinorVersion( )
{
  return GetFrameLibraryMinorVersion( );
}

void FrameCErrorInit( FrameCError** Handle )
{
  *Handle = (FrameCError*)NULL;
}

/*=======================================================================
 * Implementation of internal functions
 *=======================================================================*/

namespace FrameC
{
  const char* UnknownError = "Unknown error";

  Pointers::
  Pointers( )
    : m_baton( MutexLock::Initialize( ) )
  {
    
  }

  void Pointers::
  Deposit( void* Ptr, pointer_type Type )
  {
    MutexLock	l( m_baton );

    pointer_container_type::mapped_type& v( m_pointers[ Ptr ] );
    v.s_type = Type;
  }

  void Pointers::
  Validate( void* Ptr, pointer_type Type )
  {
    MutexLock	l( m_baton );

    pointer_container_type::const_iterator
      cur = m_pointers.find( Ptr );

    if ( cur == m_pointers.end( ) )
    {
      std::ostringstream	msg;

      msg << "Invalid pointer: nothing is known about the pointer";

      throw std::invalid_argument( msg.str( ) );
    }
    else if ( cur->second.s_type != Type )
    {
      std::ostringstream	msg;

      msg << "Invalid pointer: Expected type: " << type_name( Type )
	  << " but received type: " << type_name( cur->second.s_type )
	;

      throw std::invalid_argument( msg.str( ) );
    }
  }

  void Pointers::
  Withdraw( void* Ptr )
  {
    MutexLock	l( m_baton );

    try
    {
      m_pointers.erase( Ptr );
    }
    catch( ... )
    {
    }
  }

  Pointers	PointerHash;
}

#endif /* FRAME_C_CC */
