#ifndef FRAME_CPP_INTERFACE__FRAME_C_H
#define FRAME_CPP_INTERFACE__FRAME_C_H

#include "framecpp/config.h"

#include "general/types.hh"

#ifdef __cplusplus
#define restrict const
extern "C" {
#endif /* __cplusplus */

  /**
   *
   */
  struct gpstime {
    INT_4U	sec;
    INT_4U	nan;
  };
  typedef struct gpstime gpstime_t;

  /**
   * \brief Enumeration of error codes
   *
   * To aid in error handling, the C interface adopts from the errno
   * paradigm and has this enumerated type to describe numerically
   * the classification of the error that occurred.
   */
  typedef enum errno_ {
    /**
     * \brief No error
     *
     * This enumerated type is used to represent that the call
     * completed successfully
     */
    FRAMEC_ERRNO_NO_ERROR = 0,
    /**
     * \brief Error allocating the memory associated with the detector structure.
     */
    FRAMEC_ERRNO_FR_DETECTOR_ALLOC_ERROR,
    /**
     * \brief Error releasing the memory associated with the detector structure.
     */
    FRAMEC_ERRNO_FR_DETECTOR_FREE_ERROR,
    /**
     * \brief Error reading the detector structure from the frame file.
     */
    FRAMEC_ERRNO_FR_DETECTOR_READ_ERROR,
    /**
     * \brief Error writing the detector structure to the frame file.
     */
    FRAMEC_ERRNO_FR_DETECTOR_WRITE_ERROR,
    /**
     * \brief Error opening the frame file
     */
    FRAMEC_ERRNO_FRAME_OPEN_ERROR,
    /**
     * \brief Error closing the frame file
     */
    FRAMEC_ERRNO_FRAME_CLOSE_ERROR,
    /**
     * \brief Error obtaining the memory needed for the FrFrame structure.
     */
    FRAMEC_ERRNO_FR_FRAME_ALLOC_ERROR,
    /**
     * \brief Error releasing the memory associated with the FrFrame structure.
     */
    FRAMEC_ERRNO_FR_FRAME_FREE_ERROR,
    /**
     * \brief Error reading the FrFrame structure from the frame file.
     */
    FRAMEC_ERRNO_FR_FRAME_READ_ERROR,
    /**
     * \brief Error writing the FrFrame structure to the frame file.
     */
    FRAMEC_ERRNO_FR_FRAME_WRITE_ERROR,
    /**
     * \brief Error releasing the memory associated with the FrTOC structure.
     */
    FRAMEC_ERRNO_FR_TOC_FREE_ERROR,
    /**
     * \brief Error reading the FrTOC structure from the frame file.
     */
    FRAMEC_ERRNO_FR_TOC_READ_ERROR
  } framec_errno_type;
  
  /**
   * \brief Information concerning failure of a function
   */
  typedef struct FrameCError_ {
    framec_errno_type	s_errno;
    const char*		s_message;
  } FrameCError;

  void FrameCErrorInit( FrameCError** Handle );

  /**
   * \brief Return the minor version number of the frame library.
   *
   * \return
   *     The minor version number of the frame software.
   *     This value is assiged by the developer of the frame library
   *     and is only meaningful when compared to other values for the
   *     same frame library.
   */
  INT_2U FrameCLibraryMinorVersion( );

#ifdef __cplusplus
} /* extern "C" */
#endif /* __cplusplus */

#endif /* FRAME_CPP_INTERFACE__FRAME_C_H */
