#ifndef FrameCPP_VERSION_3_FrMsg_HH
#define FrameCPP_VERSION_3_FrMsg_HH

#include <vector>

#include "general/types.hh"

#include "framecpp/Common/TOCInfo.hh"

#include "framecpp/Version3/FrameSpec.hh"
#include "framecpp/Version3/STRING.hh"

namespace FrameCPP
{
  namespace Version_3
  {
    class FrMsg
      : public Common::FrameSpec::Object,
	public Common::TOCInfo
    {
    public:
      //: Default constructor
      FrMsg( );

      FrMsg( const FrMsg& Source );

      FrMsg( const std::string& Alarm,
	     const std::string& Message,
	     INT_4U Severity );

      static const char* const StructName( );

      static const Common::Description* StructDescription( );

      //: Number of bytes needed to write this structure
      cmn_streamsize_type Bytes( const Common::StreamBase& Stream) const;

      virtual FrMsg* Create( Common::IStream& Stream ) const;

      virtual const char* const ObjectStructName( ) const;

      virtual void Write( Common::OStream& Stream ) const;

      virtual void TOCQuery( int InfoClass, ... ) const;

      //: Accessors
      const std::string& GetAlarm() const;
      const std::string& GetMessage( ) const { return m_data.message; }
      const INT_4U GetSeverity( ) const { return m_data.severity; }
   
      //: Merge with another FrAdcData
      FrMsg& Merge( const FrMsg& RHS );

      //-----------------------------------------------------------------
      /// \brief Promote object
      ///
      /// \param[in] Source
      /// \param[in] Obj
      /// \param[in] Stream
      ///
      /// \return
      ///     Promoted version of Obj.
      //-----------------------------------------------------------------
      static std::auto_ptr< Object >
      Promote( INT_2U Source,
	       Object* Obj, Common::IStream* Stream );

      /// \brief comparison operator
      bool operator==( const FrMsg& RHS ) const;

      /// \brief comparison operator
      virtual bool operator==( const Common::FrameSpec::Object& Obj ) const;

    protected:
      virtual std::auto_ptr< Object >
      demote( INT_2U Target, Common::IStream* Stream ) const;

      virtual std::auto_ptr< Object >
      promote( INT_2U Target, Object* Obj, Common::IStream* Stream ) const;


    private:
      using Common::FrameSpec::Object::Create;

      struct fr_msg_data_type {
	bool operator==( const fr_msg_data_type& RHS ) const;

	STRING	alarm;
	STRING	message;
	INT_4U	severity;
      };

      fr_msg_data_type	m_data;

      FrMsg( Common::IStream& Stream );
    };

    inline const char* const FrMsg::
    StructName( )
    {
      static const char* const name = "FrMsg";
      return name;
    }

    inline bool FrMsg::
    operator==( const FrMsg& RHS ) const
    {
      return ( m_data == RHS.m_data );
    }

  } // namespace - Version_3
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_3_FrMsg_HH */
