#ifndef FrameCPP_VERSION_4_FrSimData_HH
#define FrameCPP_VERSION_4_FrSimData_HH

#include <vector>

#include "framecpp/Common/SearchContainer.hh"
#include "framecpp/Common/TOCInfo.hh"

#include "framecpp/Version3/FrSimData.hh"

#include "framecpp/Version4/FrameSpec.hh"
#include "framecpp/Version4/STRING.hh"

#include "framecpp/Version4/FrHistory.hh"
#include "framecpp/Version4/FrTable.hh"
#include "framecpp/Version4/FrVect.hh"

namespace FrameCPP
{
  namespace Version_4
  {
    //===================================================================
    // FrSimDataStorage
    //===================================================================

    typedef Previous::FrSimDataStorage FrSimDataStorage;

    //===================================================================
    // FrSimDataRefs
    //===================================================================
    template< typename FrVect, typename FrTable >
    class FrSimDataRefsT
    {
    public:
      static const INT_2U MAX_REF = 4;

      typedef typename Common::SearchContainer< FrVect, &FrVect::GetName >
      data_type;
      typedef typename data_type::iterator iterator;
      typedef typename data_type::const_iterator const_iterator;

      typedef typename Common::SearchContainer< FrVect, &FrVect::GetName >
      input_type;
      typedef typename input_type::iterator input_iterator;
      typedef typename input_type::const_iterator const_input_iterator;

      typedef typename Common::SearchContainer< FrTable, &FrTable::GetNameSlow >
      table_type;
      typedef typename table_type::iterator table_iterator;
      typedef typename table_type::const_iterator const_table_iterator;

      FrSimDataRefsT( )
      {
      }

      FrSimDataRefsT( const FrSimDataRefsT& Source )
      {
	m_refs.data = Source.m_refs.data;
	m_refs.input = Source.m_refs.input;
	m_refs.table = Source.m_refs.table;
      }

      inline const data_type&
      RefData( ) const
      {
	return m_refs.data;
      }

      inline data_type&
      RefData( )
      {
	return m_refs.data;
      }

      inline const input_type&
      RefInput( ) const
      {
	return m_refs.input;
      }

      inline input_type&
      RefInput( )
      {
	return m_refs.input;
      }

      inline const table_type&
      RefTable( ) const
      {
	return m_refs.table;
      }

      inline table_type&
      RefTable( )
      {
	return m_refs.table;
      }

    protected:
      struct ref_type
      {
	data_type	data;
	input_type	input;
	table_type	table;

	inline FrameCPP::cmn_streamsize_type
	Bytes( const Common::StreamBase& Stream ) const
	{
	  return
	    Stream.PtrStructBytes( )	// data
	    + Stream.PtrStructBytes( )	// input
	    + Stream.PtrStructBytes( )	// table
	    ;
	}

	inline bool
	operator==( const ref_type& RHS ) const
	{
	  return ( ( this == &RHS )
		   || ( ( data == RHS.data )
			&& ( input == RHS.input )
			&& ( table == RHS.table )
			) );
	}
	
	void 
	operator( )( Common::IStream& Stream )
	{
	  Stream >> data
		 >> input
		 >> table
	    ;
	}

	void 
	operator( )( Common::OStream& Stream ) const
	{
	  Stream << data
		 << input
		 << table
	    ;
	}
      };

      ref_type	m_refs;
    };

    typedef FrSimDataRefsT< FrVect, FrTable> FrSimDataRefs;

    //===================================================================
    //===================================================================
    class FrSimData
      : public Common::FrameSpec::Object,
	public FrSimDataStorage,
	public FrSimDataRefs,
	public Common::TOCInfo
    {
    public:
      //: Default constructor
      FrSimData( );
      FrSimData( const FrSimData& SimData );
      FrSimData( const std::string& Name,
		 const std::string& Comment,
		 REAL_4 SampleRate );
      FrSimData( Previous::FrSimData& Source,
		 Common::IStream* Stream );

      static std::auto_ptr< Object >
      Promote( INT_2U Source,
	       Object* Obj, Common::IStream* Stream );

      static const char* const StructName( );

      static const Common::Description* StructDescription( );

      //: Number of bytes needed to write this structure
      cmn_streamsize_type Bytes( const Common::StreamBase& Stream) const;

      virtual FrSimData* Create( Common::IStream& Stream ) const;

      virtual const char* const ObjectStructName( ) const;

      virtual void Write( Common::OStream& Stream ) const;

      /// \brief Retrive the name
      const std::string& GetNameSlow() const;
      
      //: Merge with another FrAdcData
      FrSimData& Merge( const FrSimData& RHS );

      /// \brief comparison operator
      bool operator==( const FrSimData& RHS ) const;

      /// \brief comparison operator
      virtual bool operator==( const Common::FrameSpec::Object& Obj ) const;

      virtual void TOCQuery( int InfoClass, ... ) const;

    protected:
      /// \brief Down grade an object
      virtual std::auto_ptr< Object >
      demote( INT_2U Target, Common::IStream* Stream ) const;

      /// \brief Upgrade an object
      virtual std::auto_ptr< Object >
      promote( INT_2U Target, Object* Obj, Common::IStream* Stream ) const;

    private:
      using Common::FrameSpec::Object::Create;

      FrSimData( Common::IStream& Stream );
    };

    inline std::auto_ptr< Common::FrameSpec::Object > FrSimData::
    Promote( INT_2U Source,
	     Common::FrameSpec::Object* Obj, Common::IStream* Stream )
    {
      return Object::PromoteObject< Previous::FrSimData, FrSimData >
	( DATA_FORMAT_VERSION, Source, Obj, Stream );
    }

    inline const char* const FrSimData::
    StructName( )
    {
      static const char* const name = "FrSimData";
      return name;
    }

    inline bool FrSimData::
    operator==( const FrSimData& RHS ) const
    {
      return ( ( this == &RHS )
	       || ( ( m_data == RHS.m_data )
		    && ( m_refs == RHS.m_refs )
		    ) );
    }
  } // namespace - Version_4
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_4_FrSimData_HH */
